@@ -62,6 +62,11 @@ class FileOrSequence( object ):
             self.__class__.__name__, repr( self.fos ) )   
             
    def get_line_number_string( self ):
+      if self.line_no is None:
+         if isinstance( self.fos, str ):
+            return "file %s closed" % self.fos
+	 else:
+            return "file closed"
       if isinstance( self.fos, str ):
          return "line %d of file %s" % ( self.line_no, self.fos )
       else:
@@ -123,7 +128,7 @@ class GenomicFeature( object ):
       except AttributeError:
          source = "."
       try:
-         source = self.score
+         score = self.score
       except AttributeError:
          score = "."
       try:
@@ -138,13 +143,13 @@ class GenomicFeature( object ):
          sep = "="
       else:
          sep = " "
-      attr_str = '; '.join( [ '%s%s\"%s\"' % ( ak, sep, urllib.quote( attr[ak] ) ) for ak in attr ] )
+      attr_str = '; '.join( [ '%s%s\"%s\"' % ( ak, sep, attr[ak] ) for ak in attr ] )
       return "\t".join( str(a) for a in ( self.iv.chrom, source, 
-         self.type, self.iv.start, self.iv.end, score, 
+         self.type, self.iv.start+1, self.iv.end, score, 
          self.iv.strand, frame, attr_str ) ) + "\n"
          
 
-_re_attr_main = re.compile( "\s*(\w+)[\s=]+(.*)" )
+_re_attr_main = re.compile( "\s*([^\s\=]+)[\s=]+(.*)" )
 _re_attr_empty = re.compile( "^\s*$" )
 
 def parse_GFF_attribute_string( attrStr, extra_return_first_value=False ):
@@ -167,7 +172,7 @@ def parse_GFF_attribute_string( attrStr, extra_return_first_value=False ):
       val = mo.group(2)
       if val.startswith( '"' ) and val.endswith( '"' ):
          val = val[1:-1]
-      val = urllib.unquote( val )
+      #val = urllib.unquote( val )
       d[ intern(mo.group(1)) ] = intern(val)
       if extra_return_first_value and i == 0:
          first_val = val         
@@ -176,6 +181,7 @@ def parse_GFF_attribute_string( attrStr, extra_return_first_value=False ):
    else:
       return d
 
+_re_gff_meta_comment = re.compile( "##\s*(\S+)\s+(\S*)" )
 
 class GFF_Reader( FileOrSequence ):
 
@@ -191,11 +197,16 @@ class GFF_Reader( FileOrSequence ):
    def __init__( self, filename_or_sequence, end_included=False ):
       FileOrSequence.__init__( self, filename_or_sequence )
       self.end_included = end_included
+      self.metadata = {}
    
    
    def __iter__( self ):
       for line in FileOrSequence.__iter__( self ):
          if line.startswith( '#' ):
+            if line.startswith( "##" ):
+               mo = _re_gff_meta_comment.match( line )
+               if mo:
+                  self.metadata[ mo.group(1) ] = mo.group(2)
             continue
          ( seqname, source, feature, start, end, score, 
             strand, frame, attributeStr ) = line.split( "\t", 8 )   
@@ -423,7 +434,7 @@ class SolexaExportReader( FileOrSequence ):
       if solexa_old:
          self.qualscale = "solexa-old"
       else:
-         self.qualscale = "solexa-1.3"
+         self.qualscale = "solexa"
 
    @classmethod
    def parse_line_bare( dummy, line ):
@@ -542,13 +553,11 @@ def pair_SAM_alignments( alignments ):
                ( read1.pe_which == "unknown" and read2.pe_which == "unknown" ) ):
          warnings.warn( "Incorrect first/second assignments in mate pairs " + 
             read1.read.name )
-      if not ( read1.proper_pair and read2.proper_pair ):
-         warnings.warn( "Incorrect 'proper_pair' flag value for read pair " + 
-            read1.read.name )
-      if not ( read1.mate_start == read2.iv.start_as_pos and 
-            read2.mate_start == read1.iv.start_as_pos ):
-         warnings.warn( "Read pair " + read1.read.name +
-            " show inconsistency between 'iv' and 'mate_start' values" )
+      if read1.aligned and read2.aligned:
+	 if not ( read1.mate_start == read2.iv.start_as_pos and 
+               read2.mate_start == read1.iv.start_as_pos ):
+            warnings.warn( "Read pair " + read1.read.name +
+               " show inconsistency between 'iv' and 'mate_start' values" )
       return ( read1, read2 )
 
    def process_single_read( read ):
